@@ -64,8 +64,8 @@ module Sexp_style = struct
 end
 
 module Phys_equal (M : sig
-    type t [@@deriving sexp_of]
-  end) =
+  type t [@@deriving sexp_of]
+end) =
 struct
   type t = M.t [@@deriving sexp_of]
 
@@ -86,15 +86,15 @@ let hide_positions_in_string =
             doesn't implement back-references, precluding a simple substitution.  Instead,
             we provide a length of matched data to copy into the output, effectively acting
             like a back-reference in this special case. *)
-        "[a-zA-z]:[0-9]+:[0-9]+", 1, ":LINE:COL"
-      ; "line [0-9]+:", 0, "line LINE:"
-      ; "line [0-9]+, characters [0-9]+-[0-9]+", 0, "line LINE, characters C1-C2"
-      ]
-        |> List.map ~f:(fun (pattern, prefix_len, expansion) ->
-          let rex = Re.regexp pattern in
-          fun string ->
-            Re.substitute ~rex string ~subst:(fun orig ->
-              String.concat [ String.prefix orig prefix_len; expansion ])))
+         "[a-zA-z]:[0-9]+:[0-9]+", 1, ":LINE:COL"
+       ; "line [0-9]+:", 0, "line LINE:"
+       ; "line [0-9]+, characters [0-9]+-[0-9]+", 0, "line LINE, characters C1-C2"
+       ]
+       |> List.map ~f:(fun (pattern, prefix_len, expansion) ->
+            let rex = Re.regexp pattern in
+            fun string ->
+              Re.substitute ~rex string ~subst:(fun orig ->
+                String.concat [ String.prefix orig prefix_len; expansion ])))
   in
   fun string ->
     List.fold (force expanders) ~init:string ~f:(fun ac expander -> expander ac)
@@ -124,12 +124,11 @@ let rec replace_s (sexp : Sexp.t) ~pattern ~with_ : Sexp.t =
   | List list -> List (List.map list ~f:(replace_s ~pattern ~with_))
 ;;
 
-let expect_test_output here =
-  Expect_test_collector.save_and_return_output
-    (Expect_test_common.File.Location.of_source_code_position here)
+let expect_test_output (here : Lexing.position) =
+  Ppx_expect_runtime.For_external.read_current_test_output_exn ~here
 ;;
 
-let am_running_expect_test = Expect_test_collector.am_running_expect_test
+let am_running_expect_test = Ppx_expect_runtime.For_external.am_running_expect_test
 
 let assert_am_running_expect_test here =
   if am_running_expect_test ()
@@ -155,8 +154,16 @@ let rec smash_sexp sexp ~f =
 
 let remove_backtraces =
   let prefixes =
-    (* taken from [printexc.ml] in ocaml runtime *)
-    [ "Raised at "; "Re-raised at "; "Raised by primitive operation at "; "Called from " ]
+    List.concat
+      [ (* taken from [printexc.ml] in ocaml runtime *)
+        [ "Raised at "
+        ; "Re-raised at "
+        ; "Raised by primitive operation at "
+        ; "Called from "
+        ]
+        (* based on [monitor.ml] in async_kernel *)
+      ; [ "Caught by monitor " ]
+      ]
   in
   smash_sexp ~f:(function
     | Sexp.(List (Atom hd :: _))
@@ -171,11 +178,12 @@ let print_s ?hide_positions sexp = print_string (sexp_to_string ?hide_positions
 let on_print_cr = ref (fun string -> print_endline string)
 
 let print_cr_with_optional_message
-      ?(cr = CR.CR)
-      ?(hide_positions = CR.hide_unstable_output cr)
-      here
-      optional_message
+  ?(cr = CR.CR)
+  ?(hide_positions = CR.hide_unstable_output cr)
+  here
+  optional_message
   =
+  assert_am_running_expect_test here;
   match cr with
   | Suppress -> ()
   | _ ->
@@ -203,15 +211,15 @@ let require ?cr ?hide_positions ?if_false_then_print_s here bool =
 ;;
 
 let require_equal
-      (type a)
-      ?cr
-      ?hide_positions
-      ?if_false_then_print_s
-      ?(message = "values are not equal")
-      here
-      (module M : With_equal with type t = a)
-      x
-      y
+  (type a)
+  ?cr
+  ?hide_positions
+  ?if_false_then_print_s
+  ?(message = "values are not equal")
+  here
+  (module M : With_equal with type t = a)
+  x
+  y
   =
   require
     ?cr
@@ -228,15 +236,15 @@ let require_equal
 ;;
 
 let require_not_equal
-      (type a)
-      ?cr
-      ?hide_positions
-      ?if_false_then_print_s
-      ?(message = "values are equal")
-      here
-      (module M : With_equal with type t = a)
-      x
-      y
+  (type a)
+  ?cr
+  ?hide_positions
+  ?if_false_then_print_s
+  ?(message = "values are equal")
+  here
+  (module M : With_equal with type t = a)
+  x
+  y
   =
   require
     ?cr
@@ -253,14 +261,14 @@ let require_not_equal
 ;;
 
 let require_compare_equal
-      (type a)
-      ?cr
-      ?hide_positions
-      ?message
-      here
-      (module M : With_compare with type t = a)
-      x
-      y
+  (type a)
+  ?cr
+  ?hide_positions
+  ?message
+  here
+  (module M : With_compare with type t = a)
+  x
+  y
   =
   require_equal
     ?cr
@@ -277,14 +285,14 @@ let require_compare_equal
 ;;
 
 let require_compare_not_equal
-      (type a)
-      ?cr
-      ?hide_positions
-      ?message
-      here
-      (module M : With_compare with type t = a)
-      x
-      y
+  (type a)
+  ?cr
+  ?hide_positions
+  ?message
+  here
+  (module M : With_compare with type t = a)
+  x
+  y
   =
   require_not_equal
     ?cr
@@ -301,13 +309,13 @@ let require_compare_not_equal
 ;;
 
 let require_sets_are_equal
-      (type elt)
-      ?cr
-      ?hide_positions
-      ?(names = "first", "second")
-      here
-      first
-      second
+  (type elt)
+  ?cr
+  ?hide_positions
+  ?(names = "first", "second")
+  here
+  first
+  second
   =
   let module Elt = struct
     type t = elt
@@ -365,21 +373,21 @@ let require_does_not_raise ?cr ?hide_positions ?show_backtrace here f =
   | Raised message -> print_cr ?cr ?hide_positions here message
 ;;
 
-let require_does_raise ?cr ?hide_positions ?show_backtrace here f =
+let require_does_raise ?cr ?(hide_positions = false) ?show_backtrace here f =
   match try_with f ?show_backtrace with
-  | Raised message -> print_s ?hide_positions message
-  | Did_not_raise -> print_cr ?cr ?hide_positions here [%message "did not raise"]
+  | Raised message -> print_s ~hide_positions message
+  | Did_not_raise -> print_cr ?cr ~hide_positions here [%message "did not raise"]
 ;;
 
 let require_first_gen
-      (type first second)
-      ?cr
-      ?hide_positions
-      ?(print_first : (first -> Sexp.t) option)
-      ~message
-      here
-      (sexp_of_second : second -> Sexp.t)
-      (either : (first, second) Either.t)
+  (type first second)
+  ?cr
+  ?hide_positions
+  ?(print_first : (first -> Sexp.t) option)
+  ~message
+  here
+  (sexp_of_second : second -> Sexp.t)
+  (either : (first, second) Either.t)
   =
   match either with
   | First first ->
@@ -464,13 +472,13 @@ let require_error ?cr ?hide_positions ?(print_error = false) here sexp_of_ok res
 ;;
 
 let print_and_check_round_trip
-      (type a)
-      ?cr
-      ?hide_positions
-      here
-      (module T : With_equal with type t = a)
-      reprs
-      examples
+  (type a)
+  ?cr
+  ?hide_positions
+  here
+  (module T : With_equal with type t = a)
+  reprs
+  examples
   =
   require_does_not_raise ?cr ?hide_positions here (fun () ->
     let tag_of name =
@@ -516,12 +524,12 @@ let print_and_check_round_trip
 ;;
 
 let print_and_check_stringable
-      (type a)
-      ?cr
-      ?hide_positions
-      here
-      (module T : With_stringable with type t = a)
-      list
+  (type a)
+  ?cr
+  ?hide_positions
+  here
+  (module T : With_stringable with type t = a)
+  list
   =
   let module T = struct
     include T
@@ -542,12 +550,12 @@ let print_and_check_stringable
 ;;
 
 let print_and_check_sexpable
-      (type a)
-      ?cr
-      ?hide_positions
-      here
-      (module T : With_sexpable with type t = a)
-      list
+  (type a)
+  ?cr
+  ?hide_positions
+  here
+  (module T : With_sexpable with type t = a)
+  list
   =
   let module Conv = struct
     type t = T.t
@@ -570,14 +578,14 @@ let show_raise (type a) ?hide_positions ?show_backtrace (f : unit -> a) =
 ;;
 
 let quickcheck_m
-      (type a)
-      here
-      ?config
-      ?cr
-      ?examples
-      ?hide_positions
-      (module M : Base_quickcheck.Test.S with type t = a)
-      ~f
+  (type a)
+  here
+  ?config
+  ?cr
+  ?examples
+  ?hide_positions
+  (module M : Base_quickcheck.Test.S with type t = a)
+  ~f
   =
   Base_quickcheck.Test.result
     ?config
@@ -591,24 +599,24 @@ let quickcheck_m
         require_does_not_raise here ?cr ?hide_positions (fun () -> f elt));
       if Queue.is_empty crs then Ok () else Error (Queue.to_list crs))
   |> Result.iter_error ~f:(fun (input, output) ->
-    print_s [%message "quickcheck: test failed" (input : M.t)];
-    List.iter output ~f:print_endline)
+       print_s [%message "quickcheck: test failed" (input : M.t)];
+       List.iter output ~f:print_endline)
 ;;
 
 let quickcheck
-      (type a)
-      here
-      ?cr
-      ?hide_positions
-      ?(seed = Base_quickcheck.Test.default_config.seed)
-      ?(sizes = Base_quickcheck.Test.default_config.sizes)
-      ?(trials = Base_quickcheck.Test.default_config.test_count)
-      ?(shrinker = Base_quickcheck.Shrinker.atomic)
-      ?(shrink_attempts = Base_quickcheck.Test.default_config.shrink_count)
-      ?examples
-      ~sexp_of
-      ~f
-      quickcheck_generator
+  (type a)
+  here
+  ?cr
+  ?hide_positions
+  ?(seed = Base_quickcheck.Test.default_config.seed)
+  ?(sizes = Base_quickcheck.Test.default_config.sizes)
+  ?(trials = Base_quickcheck.Test.default_config.test_count)
+  ?(shrinker = Base_quickcheck.Shrinker.atomic)
+  ?(shrink_attempts = Base_quickcheck.Test.default_config.shrink_count)
+  ?examples
+  ~sexp_of
+  ~f
+  quickcheck_generator
   =
   quickcheck_m
     here
@@ -625,3 +633,181 @@ let quickcheck
     end)
     ~f
 ;;
+
+module Tuple2 (M : Quickcheck.Test.S) : Quickcheck.Test.S with type t = M.t * M.t = struct
+  type t = M.t * M.t [@@deriving sexp_of]
+
+  let quickcheck_generator = [%quickcheck.generator: M.t * M.t]
+  let quickcheck_shrinker = [%quickcheck.shrinker: M.t * M.t]
+end
+
+module Tuple3 (M : Quickcheck.Test.S) : Quickcheck.Test.S with type t = M.t * M.t * M.t =
+struct
+  type t = M.t * M.t * M.t [@@deriving sexp_of]
+
+  let quickcheck_generator = [%quickcheck.generator: M.t * M.t * M.t]
+  let quickcheck_shrinker = [%quickcheck.shrinker: M.t * M.t * M.t]
+end
+
+let test_compare here ?config ?cr ?hide_positions (module M : With_quickcheck_and_compare)
+  =
+  let check bool msg = require here ?cr ?hide_positions bool ~if_false_then_print_s:msg in
+  let check_reflexive x =
+    let compare_x_x = M.compare x x in
+    check
+      (compare_x_x = 0)
+      [%lazy_message "[compare] is not reflexive" (x : M.t) (compare_x_x : int)]
+  in
+  let check_asymmetric x y =
+    let compare_x_y = M.compare x y in
+    let compare_y_x = M.compare y x in
+    check
+      (Sign.equal (Sign.of_int compare_x_y) (Sign.flip (Sign.of_int compare_y_x)))
+      [%lazy_message
+        "[compare] is not asymmetric"
+          (x : M.t)
+          (y : M.t)
+          (compare_x_y : int)
+          (compare_y_x : int)]
+  in
+  let check_transitive x y z =
+    let compare_x_y = M.compare x y in
+    let compare_y_z = M.compare y z in
+    let compare_x_z = M.compare x z in
+    let expected : Ordering.t option =
+      match Ordering.of_int compare_x_y, Ordering.of_int compare_y_z with
+      | Equal, Equal -> Some Equal
+      | Less, Less | Less, Equal | Equal, Less -> Some Less
+      | Greater, Greater | Greater, Equal | Equal, Greater -> Some Greater
+      | Less, Greater | Greater, Less -> None
+    in
+    Option.iter expected ~f:(fun expected ->
+      check
+        (Ordering.equal expected (Ordering.of_int compare_x_z))
+        [%lazy_message
+          "[compare] is not transitive"
+            (x : M.t)
+            (y : M.t)
+            (z : M.t)
+            (compare_x_y : int)
+            (compare_y_z : int)
+            (compare_x_z : int)])
+  in
+  let test m ~f = quickcheck_m here ?config ?cr ?hide_positions m ~f in
+  test
+    (module M)
+    ~f:(fun x ->
+      check_reflexive x;
+      check_asymmetric x x;
+      check_transitive x x x);
+  test
+    (module Tuple2 (M))
+    ~f:(fun (x, y) ->
+      check_asymmetric x y;
+      check_transitive x x y;
+      check_transitive x y x;
+      check_transitive y x x);
+  test
+    (module Tuple3 (M))
+    ~f:(fun (x, y, z) ->
+      check_transitive x y z;
+      check_transitive x z y;
+      check_transitive y x z)
+;;
+
+let test_equal here ?config ?cr ?hide_positions (module M : With_quickcheck_and_equal) =
+  let check bool msg = require here ?cr ?hide_positions bool ~if_false_then_print_s:msg in
+  let check_reflexive x =
+    let equal_x_x = M.equal x x in
+    check
+      equal_x_x
+      [%lazy_message "[equal] is not reflexive" (x : M.t) (equal_x_x : bool)]
+  in
+  let check_symmetric x y =
+    let equal_x_y = M.equal x y in
+    let equal_y_x = M.equal y x in
+    check
+      (Bool.equal equal_x_y equal_y_x)
+      [%lazy_message
+        "[equal] is not symmetric"
+          (x : M.t)
+          (y : M.t)
+          (equal_x_y : bool)
+          (equal_y_x : bool)]
+  in
+  let check_transitive x y z =
+    let equal_x_y = M.equal x y in
+    let equal_y_z = M.equal y z in
+    let equal_x_z = M.equal x z in
+    let msg =
+      [%lazy_message
+        "[equal] is not transitive"
+          (x : M.t)
+          (y : M.t)
+          (z : M.t)
+          (equal_x_y : bool)
+          (equal_y_z : bool)
+          (equal_x_z : bool)]
+    in
+    match equal_x_y, equal_y_z with
+    | true, true -> check equal_x_z msg
+    | true, false | false, true -> check (not equal_x_z) msg
+    | false, false -> ()
+  in
+  let test m ~f = quickcheck_m here ?config ?cr ?hide_positions m ~f in
+  test
+    (module M)
+    ~f:(fun x ->
+      check_reflexive x;
+      check_symmetric x x;
+      check_transitive x x x);
+  test
+    (module Tuple2 (M))
+    ~f:(fun (x, y) ->
+      check_symmetric x y;
+      check_transitive x x y;
+      check_transitive x y x;
+      check_transitive y x x);
+  test
+    (module Tuple3 (M))
+    ~f:(fun (x, y, z) ->
+      check_transitive x y z;
+      check_transitive x z y;
+      check_transitive y x z)
+;;
+
+let test_compare_and_equal
+  here
+  ?config
+  ?cr
+  ?hide_positions
+  (module M : With_quickcheck_and_compare_and_equal)
+  =
+  test_compare here ?config ?cr ?hide_positions (module M);
+  test_equal here ?config ?cr ?hide_positions (module M);
+  let check_agreement x y =
+    let compare_x_y = M.compare x y in
+    let equal_x_y = M.equal x y in
+    require
+      here
+      ?cr
+      ?hide_positions
+      (Bool.equal equal_x_y (compare_x_y = 0))
+      ~if_false_then_print_s:
+        [%lazy_message
+          "[compare] and [equal] do not agree"
+            (x : M.t)
+            (y : M.t)
+            (compare_x_y : int)
+            (equal_x_y : bool)]
+  in
+  quickcheck_m
+    here
+    ?config
+    ?cr
+    ?hide_positions
+    (module Tuple2 (M))
+    ~f:(fun (x, y) ->
+      check_agreement x y;
+      check_agreement y x)
+;;
